@@ -1,11 +1,9 @@
 /* vi:ts=4:sw=4
  *
- * VIM - Vi IMproved
+ * VIM - Vi IMproved		by Bram Moolenaar
  *
- * Code Contributions By:	Bram Moolenaar			mool@oce.nl
- *							Tim Thompson			twitch!tjt
- *							Tony Andrews			onecom!wldrdg!tony 
- *							G. R. (Fred) Walter		watmath!watcgl!grwalter 
+ * Read the file "credits.txt" for a list of people who contributed.
+ * Read the file "uganda.txt" for copying and usage conditions.
  */
 
 /*
@@ -18,7 +16,7 @@
 #include "param.h"
 
 static void qf_free __ARGS((void));
-static char *qf_types __ARGS((int, int));
+static char_u *qf_types __ARGS((int, int));
 
 /*
  * for each error the next struct is allocated and linked in a list
@@ -28,14 +26,13 @@ struct qf_line
 	struct qf_line	*qf_next;	/* pointer to next error in the list */
 	struct qf_line	*qf_prev;	/* pointer to previous error in the list */
 	linenr_t		 qf_lnum;	/* line number where the error occurred */
-	char			*qf_mark;	/* pointer to that line (if != NULL) */
+	int				 qf_fnum;	/* file number for the line */
 	int				 qf_col;	/* column where the error occurred */
 	int				 qf_nr;		/* error number */
-	char			*qf_fname;	/* file name where the error occurred */
-	char			*qf_text;	/* description of the error */
-	char			 qf_cleared;/* set to TRUE if qf_mark has been cleared */
-	char			 qf_type;	/* type of the error (mostly 'E') */
-	char			 qf_valid;	/* valid error message detected */
+	char_u			*qf_text;	/* description of the error */
+	char_u			 qf_cleared;/* set to TRUE if line has been deleted */
+	char_u			 qf_type;	/* type of the error (mostly 'E') */
+	char_u			 qf_valid;	/* valid error message detected */
 };
 
 static struct qf_line *qf_start;		/* pointer to the first error */
@@ -44,28 +41,27 @@ static struct qf_line *qf_ptr;			/* pointer to the current error */
 static int	qf_count = 0;		/* number of errors (0 means no error list) */
 static int	qf_index;			/* current index in the error list */
 static int	qf_nonevalid;		/* set to TRUE if not a single valid entry found */
-static int	qf_marksset;		/* set to 1 when qf_mark-s have been set */
 
 /*
  * Read the errorfile into memory, line by line, building the error list.
- * Return 1 for error, 0 for success.
+ * Return FAIL for error, OK for success.
  */
 	int
 qf_init()
 {
-	char 			namebuf[CMDBUFFSIZE + 1];
-	char			errmsg[CMDBUFFSIZE + 1];
+	char_u 			namebuf[CMDBUFFSIZE + 1];
+	char_u			errmsg[CMDBUFFSIZE + 1];
 	int				col;
-	char			type;
-	char			valid;
+	int				type;
+	int				valid;
 	long			lnum;
 	int				enr;
 	FILE			*fd;
 	struct qf_line	*qfp = NULL;
 	struct qf_line	*qfprev = NULL;		/* init to make SASC shut up */
-	char			*pfmt, *fmtstr;
+	char_u			*pfmt, *fmtstr;
 #ifdef UTS2
-	char			*(adr[7]);
+	char_u			*(adr[7]);
 #else
 	void			*(adr[7]);
 #endif
@@ -76,12 +72,12 @@ qf_init()
 	if (p_ef == NULL || *p_ef == NUL)
 	{
 		emsg(e_errorf);
-		return 1;
+		return FAIL;
 	}
-	if ((fd = fopen(p_ef, "r")) == NULL)
+	if ((fd = fopen((char *)p_ef, "r")) == NULL)
 	{
-		emsg(e_openerrf);
-		return 1;
+		emsg2(e_openerrf, p_ef);
+		return FAIL;
 	}
 	qf_free();
 	qf_index = 0;
@@ -94,8 +90,8 @@ qf_init()
  */
 		/* get some space to modify the format string into */
 		/* must be able to do the largest expansion 7 times (7 x 3) */
-	maxlen = strlen(p_efm) + 25;
-	fmtstr = (char *)alloc(maxlen);
+	maxlen = STRLEN(p_efm) + 25;
+	fmtstr = alloc(maxlen);
 	if (fmtstr == NULL)
 		goto error2;
 	for (pfmt = p_efm, i = 0; *pfmt; ++pfmt, ++i)
@@ -147,25 +143,25 @@ qf_init()
 					fmtstr[i] = pfmt[1];
 					break;
 			default:
-					emsg("invalid % in format string");
+					EMSG("invalid % in format string");
 					goto error2;
 			}
 			if (adr_cnt == 7)
 			{
-				emsg("too many % in format string");
+				EMSG("too many % in format string");
 				goto error2;
 			}
 			++pfmt;
 		}
 		if (i >= maxlen - 6)
 		{
-			emsg("invalid format string");
+			EMSG("invalid format string");
 			goto error2;
 		}
 	}
 	fmtstr[i] = NUL;
 
-	while (fgets(IObuff, CMDBUFFSIZE, fd) != NULL)
+	while (fgets((char *)IObuff, CMDBUFFSIZE, fd) != NULL && !got_int)
 	{
 		if ((qfp = (struct qf_line *)alloc((unsigned)sizeof(struct qf_line))) == NULL)
 			goto error2;
@@ -179,27 +175,26 @@ qf_init()
 		type = 0;
 		valid = TRUE;
 
-		if (sscanf(IObuff, fmtstr, adr[0], adr[1], adr[2], adr[3],
+		if (sscanf((char *)IObuff, (char *)fmtstr, adr[0], adr[1], adr[2], adr[3],
 												adr[4], adr[5]) != adr_cnt)
 		{
 			namebuf[0] = NUL;			/* something failed, remove file name */
 			valid = FALSE;
-			strcpy(errmsg, IObuff);		/* copy whole line to error message */
-			if ((pfmt = strrchr(errmsg, '\n')) != NULL)
+			STRCPY(errmsg, IObuff);		/* copy whole line to error message */
+			if ((pfmt = STRRCHR(errmsg, '\n')) != NULL)
 				*pfmt = NUL;
 #ifdef MSDOS
-			if ((pfmt = strrchr(errmsg, '\r')) != NULL)
+			if ((pfmt = STRRCHR(errmsg, '\r')) != NULL)
 				*pfmt = NUL;
 #endif
 		}
 
-		if ((qfp->qf_fname = strsave(namebuf)) == NULL)
-			goto error1;
+		if (namebuf[0] == NUL)			/* no file name */
+			qfp->qf_fnum = 0;
+		else
+			qfp->qf_fnum = filelist_add(namebuf);
 		if ((qfp->qf_text = strsave(errmsg)) == NULL)
-		{
-			free(qfp->qf_fname);
 			goto error1;
-		}
 		qfp->qf_lnum = lnum;
 		qfp->qf_col = col;
 		qfp->qf_nr = enr;
@@ -217,7 +212,6 @@ qf_init()
 			qfprev->qf_next = qfp;
 		}
 		qfp->qf_next = qfp;		/* last element points to itself */
-		qfp->qf_mark = NULL;
 		qfp->qf_cleared = FALSE;
 		qfprev = qfp;
 		++qf_count;
@@ -226,6 +220,7 @@ qf_init()
 			qf_index = qf_count;
 			qf_ptr = qfp;
 		}
+		breakcheck();
 	}
 	free(fmtstr);
 	if (!ferror(fd))
@@ -239,8 +234,8 @@ qf_init()
 		else
 			qf_nonevalid = FALSE;
 		fclose(fd);
-		qf_jump(0);				/* display first error */
-		return 0;
+		qf_jump(0, 0);			/* display first error */
+		return OK;
 	}
 	emsg(e_readerrf);
 error1:
@@ -248,19 +243,22 @@ error1:
 error2:
 	fclose(fd);
 	qf_free();
-	return 1;
+	return FAIL;
 }
 
 /*
- * jump to quickfix line "errornr"; if "errornr" is zero, redisplay the same line
+ * jump to a quickfix line
+ * if dir == FORWARD go "errornr" valid entries forward
+ * if dir == BACKWARD go "errornr" valid entries backward
+ * else if "errornr" is zero, redisplay the same line
+ * else go to entry "errornr"
  */
 	void
-qf_jump(errornr)
-	int errornr;
+qf_jump(dir, errornr)
+	int		dir;
+	int		errornr;
 {
-	struct qf_line *qfp;
 	linenr_t		i;
-	char			*msgp;
 
 	if (qf_count == 0)
 	{
@@ -268,34 +266,40 @@ qf_jump(errornr)
 		return;
 	}
 
-	if (errornr == -1)		/* next valid entry */
+	if (dir == FORWARD)		/* next valid entry */
 	{
-		do
+		while (errornr--)
 		{
-			if (qf_index == qf_count)
-				break;
-			++qf_index;
-			qf_ptr = qf_ptr->qf_next;
-		} while (!qf_nonevalid && !qf_ptr->qf_valid);
+			do
+			{
+				if (qf_index == qf_count || qf_ptr->qf_next == NULL)
+					break;
+				++qf_index;
+				qf_ptr = qf_ptr->qf_next;
+			} while (!qf_nonevalid && !qf_ptr->qf_valid);
+		}
 	}
-	else if (errornr == -2)		/* previous valid entry */
+	else if (dir == BACKWARD)		/* previous valid entry */
 	{
-		do
+		while (errornr--)
 		{
-			if (qf_index == 1)
-				break;
-			--qf_index;
-			qf_ptr = qf_ptr->qf_prev;
-		} while (!qf_nonevalid && !qf_ptr->qf_valid);
+			do
+			{
+				if (qf_index == 1 || qf_ptr->qf_prev == NULL)
+					break;
+				--qf_index;
+				qf_ptr = qf_ptr->qf_prev;
+			} while (!qf_nonevalid && !qf_ptr->qf_valid);
+		}
 	}
 	else if (errornr != 0)		/* go to specified number */
 	{
-		while (errornr < qf_index && qf_index > 1)
+		while (errornr < qf_index && qf_index > 1 && qf_ptr->qf_prev != NULL)
 		{
 			--qf_index;
 			qf_ptr = qf_ptr->qf_prev;
 		}
-		while (errornr > qf_index && qf_index < qf_count)
+		while (errornr > qf_index && qf_index < qf_count && qf_ptr->qf_next != NULL)
 		{
 			++qf_index;
 			qf_ptr = qf_ptr->qf_next;
@@ -306,37 +310,24 @@ qf_jump(errornr)
 	 * If there is a file name, 
 	 * read the wanted file if needed, and check autowrite etc.
 	 */
-	if (qf_ptr->qf_fname[0] == NUL || getfile(qf_ptr->qf_fname, NULL, TRUE) <= 0)
+	if (qf_ptr->qf_fnum == 0 || filelist_getfile(qf_ptr->qf_fnum, (linenr_t)1, TRUE) == OK)
 	{
 		/*
-		 * Use mark if possible, because the line number may be invalid
-		 * after line inserts / deletes.
-		 * If qf_lnum is 0, stay on the same line.
+		 * Go to line with error, unless qf_lnum is 0.
 		 */
-		i = 0;
-		msgp = "";
-		if ((qf_ptr->qf_mark != NULL && (i = ptr2nr(qf_ptr->qf_mark, (linenr_t)0)) == 0) || qf_ptr->qf_cleared)
-			msgp = "(line changed) ";
-		if (i == 0)
-			i = qf_ptr->qf_lnum;
-		if (i > line_count)
-			i = line_count;
+		i = qf_ptr->qf_lnum;
 		if (i > 0)
-			Curpos.lnum = i;
-		Curpos.col = qf_ptr->qf_col;
-		adjustCurpos();
-		cursupdate();
-		smsg("(%d of %d) %s%s: %s", qf_index, qf_count, msgp,
-					qf_types(qf_ptr->qf_type, qf_ptr->qf_nr), qf_ptr->qf_text);
-
-		if (!qf_marksset)		/* marks not set yet: try to find them for
-									the errors in the curren file */
 		{
-			for (i = 0, qfp = qf_start; i < qf_count; ++i, qfp = qfp->qf_next)
-				if (qfp->qf_fname != NUL && strcmp(qfp->qf_fname, qf_ptr->qf_fname) == 0 && qfp->qf_lnum <= line_count && qfp->qf_lnum > 0)
-					qfp->qf_mark = nr2ptr(qfp->qf_lnum);
-			qf_marksset = 1;
+			if (i > curbuf->b_ml.ml_line_count)
+				i = curbuf->b_ml.ml_line_count;
+			curwin->w_cursor.lnum = i;
 		}
+		curwin->w_cursor.col = qf_ptr->qf_col;
+		adjust_cursor();
+		cursupdate();
+		smsg((char_u *)"(%d of %d) %s%s: %s", qf_index, qf_count, 
+					qf_ptr->qf_cleared ? (char_u *)"(line deleted) " : (char_u *)"",
+					qf_types(qf_ptr->qf_type, qf_ptr->qf_nr), qf_ptr->qf_text);
 	}
 }
 
@@ -356,26 +347,23 @@ qf_list()
 	}
 	qfp = qf_start;
 	gotocmdline(TRUE, NUL);
-#ifdef AMIGA
-	settmode(0);		/* set cooked mode so output can be halted */
-#endif
-	for (i = 1; i <= qf_count; ++i)
+	mch_start_listing();	/* may set cooked mode, so output can be halted */
+	for (i = 1; !got_int && i <= qf_count; ++i)
 	{
-		sprintf(IObuff, "%2d line %3ld col %2d %s: %s",
+		sprintf((char *)IObuff, "%2d line %3ld col %2d %s: %s",
 			i,
 			(long)qfp->qf_lnum,
 			qfp->qf_col,
 			qf_types(qfp->qf_type, qfp->qf_nr),
 			qfp->qf_text);
-		outstr(IObuff);
-		outchar('\n');
+		msg_outstr(IObuff);
+		msg_outchar('\n');
 		qfp = qfp->qf_next;
-		flushbuf();
+		flushbuf();					/* show one line at a time */
+		breakcheck();
 	}
-#ifdef AMIGA
-	settmode(1);
-#endif
-	wait_return(TRUE);
+	mch_stop_listing();
+	wait_return(FALSE);
 }
 
 /*
@@ -389,52 +377,35 @@ qf_free()
 	while (qf_count)
 	{
 		qfp = qf_start->qf_next;
-		free(qf_start->qf_fname);
 		free(qf_start->qf_text);
 		free(qf_start);
 		qf_start = qfp;
 		--qf_count;
 	}
-	qf_marksset = 0;
 }
 
 /*
- * qf_clrallmarks() - clear all marks
- *
- * Used mainly when trashing the entire buffer during ":e" type commands
- */
-	void
-qf_clrallmarks()
-{
-	int 			i;
-	struct qf_line *qfp;
-
-	if (qf_count)
-		for (i = 0, qfp = qf_start; i < qf_count; i++, qfp = qfp->qf_next)
-			qfp->qf_mark = NULL;
-	qf_marksset = 0;
-}
-
-/*
- * qf_adjustmark: set new ptr for a mark
+ * qf_mark_adjust: adjust marks
  */
    void
-qf_adjustmark(old, new)
-	char		*old, *new;
+qf_mark_adjust(line1, line2, inc)
+	linenr_t	line1;
+	linenr_t	line2;
+	long		inc;
 {
 	register int i;
 	struct qf_line *qfp;
 
 	if (qf_count)
-	{
 		for (i = 0, qfp = qf_start; i < qf_count; ++i, qfp = qfp->qf_next)
-			if (qfp->qf_mark == old)
+			if (qfp->qf_fnum == curbuf->b_fnum &&
+							qfp->qf_lnum >= line1 && qfp->qf_lnum <= line2)
 			{
-				qfp->qf_mark = new;
-				if (new == NULL)
+				if (inc == MAXLNUM)
 					qfp->qf_cleared = TRUE;
+				else
+					qfp->qf_lnum += inc;
 			}
-	}
 }
 
 /*
@@ -446,22 +417,22 @@ qf_adjustmark(old, new)
  *  w or W    n			"Warning n"
  *  other     n			"  Error n"
  */
-	static char *
+	static char_u *
 qf_types(c, nr)
 	int c, nr;
 {
-	static char	buf[20];
-	char		*p1;
+	static char_u	buf[20];
+	char_u		*p1;
 
-	p1 = "  Error";
+	p1 = (char_u *)"  Error";
 	if (c == 'W' || c == 'w')
-		p1 =  "Warning";
+		p1 =  (char_u *)"Warning";
 	else if (nr <= 0 && c != 'E' && c != 'e')
-		p1 = "";
+		p1 = (char_u *)"";
 
 	if (nr <= 0)
 		return p1;
 
-	sprintf(buf, "%s %3d", p1, nr);
+	sprintf((char *)buf, "%s %3d", p1, nr);
 	return buf;
 }
