@@ -58,6 +58,7 @@
 #include <stdio.h>
 #include "regexp.h"
 #include "regmagic.h"
+#include "param.h"
 
 /*
  * The "internal use only" fields in regexp.h are present to pass info from
@@ -136,7 +137,7 @@
  * BACK 		Normal "next" pointers all implicitly point forward; BACK
  *				exists to make loop structures possible.
  *
- * STAR,PLUS	'?', and complex '*' and '+', are implemented as circular
+ * STAR,PLUS	'=', and complex '*' and '+', are implemented as circular
  *				BRANCH structures using BACK.  Simple cases (one character
  *				per match) are implemented with STAR and PLUS for speed
  *				and to minimize recursive plunges.
@@ -172,7 +173,7 @@
 #define UCHARAT(p)		((int)*(p)&CHARBITS)
 #endif
 
-#define FAIL(m) { emsg(m); return NULL; }
+#define EMSG_RETURN(m) { emsg(m); return NULL; }
 
 static int ismult __ARGS((int));
 
@@ -180,7 +181,7 @@ static int ismult __ARGS((int));
 ismult(c)
 	int c;
 {
-	return (c == Magic('*') || c == Magic('+') || c == Magic('?'));
+	return (c == Magic('*') || c == Magic('+') || c == Magic('='));
 }
 
 /*
@@ -191,29 +192,18 @@ ismult(c)
 #define SPSTART 		04		/* Starts with * or +. */
 #define WORST			0		/* Worst case. */
 
-/*
- * The following supports the ability to ignore case in searches.
- */
-
-int 			reg_ic = 0; 	/* set by callers to ignore case */
-
-/*
- * mkup - convert to upper case IF we're doing caseless compares
- */
-#define mkup(c) 		(reg_ic ? TO_UPPER(c) : (c))
-
 /*
  * The following allows empty REs, meaning "the same as the previous RE".
  * per the ed(1) manual.
  */
 /* #define EMPTY_RE */			/* this is done outside of regexp */
 #ifdef EMTY_RE
-char		   *reg_prev_re;
+char_u		   *reg_prev_re;
 #endif
 
 #define TILDE
 #ifdef TILDE
-char		   *reg_prev_sub;
+char_u		   *reg_prev_sub;
 #endif
 
 /*
@@ -226,12 +216,12 @@ int				reg_magic = 1;
  * Global work variables for regcomp().
  */
 
-static unsigned char    *regparse;	/* Input-scan pointer. */
+static char_u    *regparse;	/* Input-scan pointer. */
 static int		regnpar;		/* () count. */
-static char 	regdummy;
-static char    *regcode;		/* Code-emit pointer; &regdummy = don't. */
+static char_u 	regdummy;
+static char_u   *regcode;		/* Code-emit pointer; &regdummy = don't. */
 static long 	regsize;		/* Code size. */
-static char   **regendp;		/* Ditto for endp. */
+static char_u   **regendp;		/* Ditto for endp. */
 
 /*
  * META contains all characters that may be magic, except '^' and '$'.
@@ -241,45 +231,80 @@ static char   **regendp;		/* Ditto for endp. */
 
 #ifdef TILDE
 # ifdef BACKREF
-       static char META[] = ".[()|?+*<>~123456789";
+       static char_u META[] = ".[()|=+*<>~123456789";
 # else
-       static char META[] = ".[()|?+*<>~";
+       static char_u META[] = ".[()|=+*<>~";
 # endif
 #else
 # ifdef BACKREF
-       static char META[] = ".[()|?+*<>123456789";
+       static char_u META[] = ".[()|=+*<>123456789";
 # else
-       static char META[] = ".[()|?+*<>";
+       static char_u META[] = ".[()|=+*<>";
 # endif
 #endif
 
 /*
  * Forward declarations for regcomp()'s friends.
  */
-static void		initchr __ARGS((unsigned char *));
+static void		initchr __ARGS((char_u *));
 static int		getchr __ARGS((void));
 static int		peekchr __ARGS((void));
 #define PeekChr() curchr	/* shortcut only when last action was peekchr() */
 static int 		curchr;
 static void		skipchr __ARGS((void));
 static void		ungetchr __ARGS((void));
-static char    *reg __ARGS((int, int *));
-static char    *regbranch __ARGS((int *));
-static char    *regpiece __ARGS((int *));
-static char    *regatom __ARGS((int *));
-static char    *regnode __ARGS((int));
-static char    *regnext __ARGS((char *));
+static char_u    *reg __ARGS((int, int *));
+static char_u    *regbranch __ARGS((int *));
+static char_u    *regpiece __ARGS((int *));
+static char_u    *regatom __ARGS((int *));
+static char_u    *regnode __ARGS((int));
+static char_u    *regnext __ARGS((char_u *));
 static void 	regc __ARGS((int));
 static void 	unregc __ARGS((void));
-static void 	reginsert __ARGS((int, char *));
-static void 	regtail __ARGS((char *, char *));
-static void 	regoptail __ARGS((char *, char *));
+static void 	reginsert __ARGS((int, char_u *));
+static void 	regtail __ARGS((char_u *, char_u *));
+static void 	regoptail __ARGS((char_u *, char_u *));
 
 #undef STRCSPN
 #ifdef STRCSPN
-static int		strcspn __ARGS((const char *, const char *));
+static int		strcspn __ARGS((const char_u *, const char_u *));
 #endif
-static int		cstrncmp __ARGS((char *, char *, int));
+
+/*
+ * skip past regular expression
+ * stop at end of 'p' of where 'dirc' is found ('/', '?', etc)
+ * take care of characters with a backslash in front of it
+ * skip strings inside [ and ].
+ */
+	char_u *
+skip_regexp(p, dirc)
+	char_u	*p;
+	int		dirc;
+{
+	int		in_range = FALSE;
+
+	for (; p[0] != NUL; ++p)
+	{
+		if (p[0] == dirc && !in_range)		/* found end of regexp */
+			break;
+		if ((p[0] == '[' && p_magic) || (p[0] == '\\' && p[1] == '[' && !p_magic))
+		{
+			in_range = TRUE;
+			if (p[0] == '\\')
+				++p;
+								/* "[^]" and "[]" are not the end of a range */
+			if (p[1] == '^')
+				++p;
+			if (p[1] == ']')
+				++p;
+		}
+		else if (p[0] == '\\' && p[1] != NUL)
+			++p;	/* skip next character */
+		else if (p[0] == ']')
+			in_range = FALSE;
+	}
+	return p;
+}
 
 /*
  - regcomp - compile a regular expression into internal code
@@ -298,17 +323,17 @@ static int		cstrncmp __ARGS((char *, char *, int));
  */
 	regexp		   *
 regcomp(exp)
-	char		   *exp;
+	char_u		   *exp;
 {
 	register regexp *r;
-	register char  *scan;
-	register char  *longest;
+	register char_u  *scan;
+	register char_u  *longest;
 	register int	len;
 	int 			flags;
 /*	extern char    *malloc();*/
 
 	if (exp == NULL) {
-		FAIL(e_null);
+		EMSG_RETURN(e_null);
 	}
 
 #ifdef EMPTY_RE			/* this is done outside of regexp */
@@ -316,13 +341,13 @@ regcomp(exp)
 		if (reg_prev_re) {
 			exp = reg_prev_re;
 		} else {
-			FAIL(e_noprevre);
+			EMSG_RETURN(e_noprevre);
 		}
 	}
 #endif
 
 	/* First pass: determine size, legality. */
-	initchr((unsigned char *)exp);
+	initchr((char_u *)exp);
 	regnpar = 1;
 	regsize = 0L;
 	regcode = &regdummy;
@@ -333,24 +358,24 @@ regcomp(exp)
 
 	/* Small enough for pointer-storage convention? */
 	if (regsize >= 32767L)		/* Probably could be 65535L. */
-		FAIL(e_toolong);
+		EMSG_RETURN(e_toolong);
 
 	/* Allocate space. */
 /*	r = (regexp *) malloc((unsigned) (sizeof(regexp) + regsize));*/
 	r = (regexp *) alloc((unsigned) (sizeof(regexp) + regsize));
 	if (r == NULL)
-		FAIL(e_outofmem);
+		EMSG_RETURN(e_outofmem);
 
 #ifdef EMPTY_RE			/* this is done outside of regexp */
 	if (exp != reg_prev_re) {
 		free(reg_prev_re);
-		if (reg_prev_re = alloc(strlen(exp) + 1))
-			strcpy(reg_prev_re, exp);
+		if (reg_prev_re = alloc(STRLEN(exp) + 1))
+			STRCPY(reg_prev_re, exp);
 	}
 #endif
 
 	/* Second pass: emit code. */
-	initchr((unsigned char *)exp);
+	initchr((char_u *)exp);
 	regnpar = 1;
 	regcode = r->program;
 	regendp = r->endp;
@@ -387,9 +412,9 @@ regcomp(exp)
 			longest = NULL;
 			len = 0;
 			for (; scan != NULL; scan = regnext(scan))
-				if (OP(scan) == EXACTLY && strlen(OPERAND(scan)) >= (size_t)len) {
+				if (OP(scan) == EXACTLY && STRLEN(OPERAND(scan)) >= (size_t)len) {
 					longest = OPERAND(scan);
-					len = strlen(OPERAND(scan));
+					len = STRLEN(OPERAND(scan));
 				}
 			r->regmust = longest;
 			r->regmlen = len;
@@ -410,14 +435,14 @@ regcomp(exp)
  * is a trifle forced, but the need to tie the tails of the branches to what
  * follows makes it hard to avoid.
  */
-	static char *
+	static char_u *
 reg(paren, flagp)
 	int 			paren;		/* Parenthesized? */
 	int 		   *flagp;
 {
-	register char  *ret;
-	register char  *br;
-	register char  *ender;
+	register char_u  *ret;
+	register char_u  *br;
+	register char_u  *ender;
 	register int	parno = 0;
 	int 			flags;
 
@@ -426,10 +451,10 @@ reg(paren, flagp)
 	/* Make an MOPEN node, if parenthesized. */
 	if (paren) {
 		if (regnpar >= NSUBEXP)
-			FAIL(e_toombra);
+			EMSG_RETURN(e_toombra);
 		parno = regnpar;
 		regnpar++;
-		ret = regnode((char)(MOPEN + parno));
+		ret = regnode(MOPEN + parno);
 		if (regendp)
 			regendp[parno] = NULL;	/* haven't seen the close paren yet */
 	} else
@@ -458,7 +483,7 @@ reg(paren, flagp)
 	}
 
 	/* Make a closing node, and hook it on the end. */
-	ender = regnode((char)((paren) ? MCLOSE + parno : END));
+	ender = regnode((paren) ? MCLOSE + parno : END);
 	regtail(ret, ender);
 
 	/* Hook the tails of the branches to the closing node. */
@@ -467,12 +492,12 @@ reg(paren, flagp)
 
 	/* Check for proper termination. */
 	if (paren && getchr() != Magic(')')) {
-		FAIL(e_toombra);
+		EMSG_RETURN(e_toombra);
 	} else if (!paren && peekchr() != '\0') {
 		if (PeekChr() == Magic(')')) {
-			FAIL(e_toomket);
+			EMSG_RETURN(e_toomket);
 		} else
-			FAIL(e_trailing);/* "Can't happen". */
+			EMSG_RETURN(e_trailing);/* "Can't happen". */
 		/* NOTREACHED */
 	}
 	/*
@@ -489,13 +514,13 @@ reg(paren, flagp)
  *
  * Implements the concatenation operator.
  */
-	static char    *
+	static char_u    *
 regbranch(flagp)
 	int 		   *flagp;
 {
-	register char  *ret;
-	register char  *chain;
-	register char  *latest;
+	register char_u  *ret;
+	register char_u  *chain;
+	register char_u  *latest;
 	int 			flags;
 
 	*flagp = WORST; 			/* Tentatively. */
@@ -520,21 +545,21 @@ regbranch(flagp)
 }
 
 /*
- - regpiece - something followed by possible [*+?]
+ - regpiece - something followed by possible [*+=]
  *
- * Note that the branching code sequences used for ? and the general cases
+ * Note that the branching code sequences used for = and the general cases
  * of * and + are somewhat optimized:  they use the same NOTHING node as
  * both the endmarker for their branch list and the body of the last branch.
  * It might seem that this node could be dispensed with entirely, but the
  * endmarker role is not redundant.
  */
-static char    *
+static char_u    *
 regpiece(flagp)
 	int 		   *flagp;
 {
-	register char  *ret;
+	register char_u  *ret;
 	register int	op;
-	register char  *next;
+	register char_u  *next;
 	int 			flags;
 
 	ret = regatom(&flags);
@@ -546,8 +571,8 @@ regpiece(flagp)
 		*flagp = flags;
 		return ret;
 	}
-	if (!(flags & HASWIDTH) && op != Magic('?'))
-		FAIL("*+ operand could be empty");
+	if (!(flags & HASWIDTH) && op != Magic('='))
+		EMSG_RETURN((char_u *)"*+ operand could be empty");
 	*flagp = (op != Magic('+')) ? (WORST | SPSTART) : (WORST | HASWIDTH);
 
 	if (op == Magic('*') && (flags & SIMPLE))
@@ -568,8 +593,8 @@ regpiece(flagp)
 		regtail(regnode(BACK), ret);	/* loop back */
 		regtail(next, regnode(BRANCH)); /* or */
 		regtail(ret, regnode(NOTHING)); /* null. */
-	} else if (op == Magic('?')) {
-		/* Emit x? as (x|) */
+	} else if (op == Magic('=')) {
+		/* Emit x= as (x|) */
 		reginsert(BRANCH, ret); /* Either x */
 		regtail(ret, regnode(BRANCH));	/* or */
 		next = regnode(NOTHING);/* null. */
@@ -578,7 +603,7 @@ regpiece(flagp)
 	}
 	skipchr();
 	if (ismult(peekchr()))
-		FAIL("Nested *?+");
+		EMSG_RETURN((char_u *)"Nested *=+");
 
 	return ret;
 }
@@ -590,11 +615,11 @@ regpiece(flagp)
  * it can turn them into a single node, which is smaller to store and
  * faster to run.
  */
-static char    *
+static char_u    *
 regatom(flagp)
 	int 		   *flagp;
 {
-	register char  *ret;
+	register char_u  *ret;
 	int 			flags;
 
 	*flagp = WORST; 			/* Tentatively. */
@@ -640,9 +665,9 @@ regatom(flagp)
 						class = UCHARAT(regparse - 2) + 1;
 						classend = UCHARAT(regparse);
 						if (class > classend + 1)
-							FAIL(e_invrange);
+							EMSG_RETURN(e_invrange);
 						for (; class <= classend; class++)
-							regc((char)class);
+							regc(class);
 						regparse++;
 					}
 				} else if (*regparse == '\\' && regparse[1]) {
@@ -653,7 +678,7 @@ regatom(flagp)
 			}
 			regc('\0');
 			if (*regparse != ']')
-				FAIL(e_toomsbra);
+				EMSG_RETURN(e_toomsbra);
 			skipchr();			/* let's be friends with the lexer again */
 			*flagp |= HASWIDTH | SIMPLE;
 		}
@@ -667,17 +692,17 @@ regatom(flagp)
 	  case '\0':
 	  case Magic('|'):
 	  case Magic(')'):
-		FAIL(e_internal);	/* Supposed to be caught earlier. */
+		EMSG_RETURN(e_internal);	/* Supposed to be caught earlier. */
 		/* break; Not Reached */
-	  case Magic('?'):
+	  case Magic('='):
 	  case Magic('+'):
 	  case Magic('*'):
-		FAIL("?+* follows nothing");
+		EMSG_RETURN((char_u *)"=+* follows nothing");
 		/* break; Not Reached */
 #ifdef TILDE
 	  case Magic('~'):			/* previous substitute pattern */
 			if (reg_prev_sub) {
-				register char *p;
+				register char_u *p;
 
 				ret = regnode(EXACTLY);
 				p = reg_prev_sub;
@@ -691,7 +716,7 @@ regatom(flagp)
 						*flagp |= SIMPLE;
 				}
 	  		} else
-				FAIL(e_nopresub);
+				EMSG_RETURN(e_nopresub);
 			break;
 #endif
 #ifdef BACKREF
@@ -714,14 +739,14 @@ regatom(flagp)
 			 * is only available in the second pass. Checking opens is
 			 * always possible.
 			 * Should also check that we don't refer to something that
-			 * is repeated (+*?): what instance of the repetition should
+			 * is repeated (+*=): what instance of the repetition should
 			 * we match? TODO.
 			 */
 			if (refnum < regnpar &&
 				(regendp == NULL || regendp[refnum] != NULL))
 				ret = regnode(BACKREF + refnum);
 			else
-				FAIL("Illegal back reference");
+				EMSG_RETURN((char_u *)"Illegal back reference");
 		}
 		break;
 #endif
@@ -740,15 +765,15 @@ regatom(flagp)
 			}
 #ifdef DEBUG
 			if (len == 0)
-				 FAIL("Unexpected magic character; check META.");
+				 EMSG_RETURN((char_u *)"Unexpected magic character; check META.");
 #endif
 			/*
-			 * If there is a following *, \+ or \? we need the character
+			 * If there is a following *, \+ or \= we need the character
 			 * in front of it as a single character operand
 			 */
 			if (len > 1 && ismult(chr))
 			{
-				unregc();			/* Back off of *+? operand */
+				unregc();			/* Back off of *+= operand */
 				ungetchr();			/* and put it back for next time */
 				--len;
 			}
@@ -766,12 +791,12 @@ regatom(flagp)
 /*
  - regnode - emit a node
  */
-static char    *				/* Location. */
+static char_u    *				/* Location. */
 regnode(op)
 	int			op;
 {
-	register char  *ret;
-	register char  *ptr;
+	register char_u  *ret;
+	register char_u  *ptr;
 
 	ret = regcode;
 	if (ret == &regdummy) {
@@ -795,7 +820,7 @@ regc(b)
 	int			b;
 {
 	if (regcode != &regdummy)
-		*(u_char *)regcode++ = b;
+		*regcode++ = b;
 	else
 		regsize++;
 }
@@ -820,11 +845,11 @@ unregc()
 static void
 reginsert(op, opnd)
 	int			op;
-	char		   *opnd;
+	char_u		   *opnd;
 {
-	register char  *src;
-	register char  *dst;
-	register char  *place;
+	register char_u  *src;
+	register char_u  *dst;
+	register char_u  *place;
 
 	if (regcode == &regdummy) {
 		regsize += 3;
@@ -847,11 +872,11 @@ reginsert(op, opnd)
  */
 static void
 regtail(p, val)
-	char		   *p;
-	char		   *val;
+	char_u		   *p;
+	char_u		   *val;
 {
-	register char  *scan;
-	register char  *temp;
+	register char_u  *scan;
+	register char_u  *temp;
 	register int	offset;
 
 	if (p == &regdummy)
@@ -870,8 +895,8 @@ regtail(p, val)
 		offset = (int)(scan - val);
 	else
 		offset = (int)(val - scan);
-	*(scan + 1) = (char) ((offset >> 8) & 0377);
-	*(scan + 2) = (char) (offset & 0377);
+	*(scan + 1) = (char_u) ((offset >> 8) & 0377);
+	*(scan + 2) = (char_u) (offset & 0377);
 }
 
 /*
@@ -879,8 +904,8 @@ regtail(p, val)
  */
 static void
 regoptail(p, val)
-	char		   *p;
-	char		   *val;
+	char_u		   *p;
+	char_u		   *val;
 {
 	/* "Operandless" and "op != BRANCH" are synonymous in practice. */
 	if (p == NULL || p == &regdummy || OP(p) != BRANCH)
@@ -899,7 +924,7 @@ static int		nextchr;	/* used for ungetchr() */
 
 static void
 initchr(str)
-unsigned char *str;
+char_u *str;
 {
 	regparse = str;
 	curchr = prevchr = nextchr = -1;
@@ -913,7 +938,7 @@ peekchr()
 		case '.':
 		case '*':
 	/*	case '+':*/
-	/*	case '?':*/
+	/*	case '=':*/
 		case '[':
 		case '~':
 			if (reg_magic)
@@ -925,15 +950,15 @@ peekchr()
 				curchr = Magic('^');
 			break;
 		case '$':
-			/* $ is only magic as the very last character */
-			if (!regparse[1])
+			/* $ is only magic as the very last character and in front of '\|' */
+			if (regparse[1] == NUL || (regparse[1] == '\\' && regparse[2] == '|'))
 				curchr = Magic('$');
 			break;
 		case '\\':
 			regparse++;
 			if (regparse[0] == NUL)
 				curchr = '\\';	/* trailing '\' */
-			else if (strchr(META, regparse[0]))
+			else if (STRCHR(META, regparse[0]))
 			{
 				/*
 				 * META contains everything that may be magic sometimes, except
@@ -1002,22 +1027,22 @@ ungetchr()
 /*
  * Global work variables for regexec().
  */
-static char    *reginput;		/* String-input pointer. */
-static char    *regbol; 		/* Beginning of input, for ^ check. */
-static char   **regstartp;		/* Pointer to startp array. */
-/* static char   **regendp;	*/	/* Ditto for endp. */
+static char_u    *reginput;		/* String-input pointer. */
+static char_u    *regbol; 		/* Beginning of input, for ^ check. */
+static char_u   **regstartp;		/* Pointer to startp array. */
+/* static char_u   **regendp;	*/	/* Ditto for endp. */
 
 /*
  * Forwards.
  */
-static int		regtry __ARGS((regexp *, char *));
-static int		regmatch __ARGS((char *));
-static int		regrepeat __ARGS((char *));
+static int		regtry __ARGS((regexp *, char_u *));
+static int		regmatch __ARGS((char_u *));
+static int		regrepeat __ARGS((char_u *));
 
 #ifdef DEBUG
 int 			regnarrate = 1;
 void			regdump __ARGS((regexp *));
-static char    *regprop __ARGS((char *));
+static char_u    *regprop __ARGS((char_u *));
 #endif
 
 /*
@@ -1026,10 +1051,10 @@ static char    *regprop __ARGS((char *));
 int
 regexec(prog, string, at_bol)
 	register regexp *prog;
-	register char  *string;
+	register char_u  *string;
 	int 			at_bol;
 {
-	register char  *s;
+	register char_u  *s;
 
 	/* Be paranoid... */
 	if (prog == NULL || string == NULL) {
@@ -1088,11 +1113,11 @@ regexec(prog, string, at_bol)
 static int						/* 0 failure, 1 success */
 regtry(prog, string)
 	regexp		   *prog;
-	char		   *string;
+	char_u		   *string;
 {
 	register int	i;
-	register char **sp;
-	register char **ep;
+	register char_u **sp;
+	register char_u **ep;
 
 	reginput = string;
 	regstartp = prog->startp;
@@ -1124,10 +1149,10 @@ regtry(prog, string)
  */
 static int						/* 0 failure, 1 success */
 regmatch(prog)
-	char		   *prog;
+	char_u		   *prog;
 {
-	register char  *scan;		/* Current node. */
-	char		   *next;		/* Next node. */
+	register char_u  *scan;		/* Current node. */
+	char_u		   *next;		/* Next node. */
 
 	scan = prog;
 #ifdef DEBUG
@@ -1170,25 +1195,25 @@ regmatch(prog)
 			break;
 		  case EXACTLY:{
 				register int	len;
-				register char  *opnd;
+				register char_u  *opnd;
 
 				opnd = OPERAND(scan);
 				/* Inline the first character, for speed. */
-				if (mkup(*opnd) != mkup(*reginput))
+				if (*opnd != *reginput && (!reg_ic || TO_UPPER(*opnd) != TO_UPPER(*reginput)))
 					return 0;
-				len = strlen(opnd);
+				len = STRLEN(opnd);
 				if (len > 1 && cstrncmp(opnd, reginput, len) != 0)
 					return 0;
 				reginput += len;
 			}
 			break;
 		  case ANYOF:
-			if (*reginput == '\0' || strchr(OPERAND(scan), *reginput) == NULL)
+			if (*reginput == '\0' || cstrchr(OPERAND(scan), *reginput) == NULL)
 				return 0;
 			reginput++;
 			break;
 		  case ANYBUT:
-			if (*reginput == '\0' || strchr(OPERAND(scan), *reginput) != NULL)
+			if (*reginput == '\0' || cstrchr(OPERAND(scan), *reginput) != NULL)
 				return 0;
 			reginput++;
 			break;
@@ -1206,7 +1231,7 @@ regmatch(prog)
 		  case MOPEN + 8:
 		  case MOPEN + 9:{
 				register int	no;
-				register char  *save;
+				register char_u  *save;
 
 				no = OP(scan) - MOPEN;
 				save = regstartp[no];
@@ -1214,16 +1239,16 @@ regmatch(prog)
 #ifdef DEBUG
 				printf("MOPEN  %d pre  @'%s' ('%s' )'%s'\n",
 					no, save,
-					regstartp[no]? regstartp[no] : "NULL",
-					regendp[no]? regendp[no] : "NULL");
+					regstartp[no] ? regstartp[no] : "NULL",
+					regendp[no] ? regendp[no] : "NULL");
 #endif
 
 				if (regmatch(next)) {
 #ifdef DEBUG
 				printf("MOPEN  %d post @'%s' ('%s' )'%s'\n",
 					no, save,
-					regstartp[no]? regstartp[no] : "NULL",
-					regendp[no]? regendp[no] : "NULL");
+					regstartp[no] ? regstartp[no] : "NULL",
+					regendp[no] ? regendp[no] : "NULL");
 #endif
 					return 1;
 				}
@@ -1241,7 +1266,7 @@ regmatch(prog)
 		  case MCLOSE + 8:
 		  case MCLOSE + 9:{
 				register int	no;
-				register char  *save;
+				register char_u  *save;
 
 				no = OP(scan) - MCLOSE;
 				save = regendp[no];
@@ -1249,16 +1274,16 @@ regmatch(prog)
 #ifdef DEBUG
 				printf("MCLOSE %d pre  @'%s' ('%s' )'%s'\n",
 					no, save,
-					regstartp[no]? regstartp[no] : "NULL",
-					regendp[no]? regendp[no] : "NULL");
+					regstartp[no] ? regstartp[no] : "NULL",
+					regendp[no] ? regendp[no] : "NULL");
 #endif
 
 				if (regmatch(next)) {
 #ifdef DEBUG
 				printf("MCLOSE %d post @'%s' ('%s' )'%s'\n",
 					no, save,
-					regstartp[no]? regstartp[no] : "NULL",
-					regendp[no]? regendp[no] : "NULL");
+					regstartp[no] ? regstartp[no] : "NULL",
+					regendp[no] ? regendp[no] : "NULL");
 #endif
 					return 1;
 				}
@@ -1293,7 +1318,7 @@ regmatch(prog)
 			break;
 #endif
 		  case BRANCH:{
-				register char  *save;
+				register char_u  *save;
 
 				if (OP(next) != BRANCH) /* No choice. */
 					next = OPERAND(scan);		/* Avoid recursion. */
@@ -1312,9 +1337,9 @@ regmatch(prog)
 			break;
 		  case STAR:
 		  case PLUS:{
-				register char	nextch;
+				register int	nextch;
 				register int	no;
-				register char  *save;
+				register char_u  *save;
 				register int	min;
 
 				/*
@@ -1323,13 +1348,19 @@ regmatch(prog)
 				 */
 				nextch = '\0';
 				if (OP(next) == EXACTLY)
-					nextch = mkup(*OPERAND(next));
+				{
+					nextch = *OPERAND(next);
+					if (reg_ic)
+						nextch = TO_UPPER(nextch);
+				}
 				min = (OP(scan) == STAR) ? 0 : 1;
 				save = reginput;
 				no = regrepeat(OPERAND(scan));
-				while (no >= min) {
+				while (no >= min)
+				{
 					/* If it could work, try it. */
-					if (nextch == '\0' || mkup(*reginput) == nextch)
+					if (nextch == '\0' || (*reginput == nextch ||
+									(reg_ic && TO_UPPER(*reginput) == nextch)))
 						if (regmatch(next))
 							return 1;
 					/* Couldn't or didn't -- back up. */
@@ -1364,33 +1395,35 @@ regmatch(prog)
  */
 static int
 regrepeat(p)
-	char		   *p;
+	char_u		   *p;
 {
 	register int	count = 0;
-	register char  *scan;
-	register char  *opnd;
+	register char_u  *scan;
+	register char_u  *opnd;
 
 	scan = reginput;
 	opnd = OPERAND(p);
 	switch (OP(p)) {
 	  case ANY:
-		count = strlen(scan);
+		count = STRLEN(scan);
 		scan += count;
 		break;
 	  case EXACTLY:
-		while (mkup(*opnd) == mkup(*scan)) {
+		while (*opnd == *scan || (reg_ic && TO_UPPER(*opnd) == TO_UPPER(*scan)))
+		{
 			count++;
 			scan++;
 		}
 		break;
 	  case ANYOF:
-		while (*scan != '\0' && strchr(opnd, *scan) != NULL) {
+		while (*scan != '\0' && cstrchr(opnd, *scan) != NULL)
+		{
 			count++;
 			scan++;
 		}
 		break;
 	  case ANYBUT:
-		while (*scan != '\0' && strchr(opnd, *scan) == NULL) {
+		while (*scan != '\0' && cstrchr(opnd, *scan) == NULL) {
 			count++;
 			scan++;
 		}
@@ -1408,9 +1441,9 @@ regrepeat(p)
 /*
  - regnext - dig the "next" pointer out of a node
  */
-static char    *
+static char_u    *
 regnext(p)
-	register char  *p;
+	register char_u  *p;
 {
 	register int	offset;
 
@@ -1436,10 +1469,10 @@ void
 regdump(r)
 	regexp		   *r;
 {
-	register char  *s;
-	register char	op = EXACTLY;		/* Arbitrary non-END op. */
-	register char  *next;
-	/*extern char    *strchr();*/
+	register char_u  *s;
+	register int	op = EXACTLY;		/* Arbitrary non-END op. */
+	register char_u  *next;
+	/*extern char_u    *strchr();*/
 
 
 	s = r->program + 1;
@@ -1476,12 +1509,12 @@ regdump(r)
 /*
  - regprop - printable representation of opcode
  */
-static char    *
+static char_u    *
 regprop(op)
-	char		   *op;
+	char_u		   *op;
 {
-	register char  *p;
-	static char 	buf[50];
+	register char_u  *p;
+	static char_u 	buf[50];
 
 	(void) strcpy(buf, ":");
 
@@ -1525,7 +1558,7 @@ regprop(op)
 	  case MOPEN + 7:
 	  case MOPEN + 8:
 	  case MOPEN + 9:
-		sprintf(buf + strlen(buf), "MOPEN%d", OP(op) - MOPEN);
+		sprintf(buf + STRLEN(buf), "MOPEN%d", OP(op) - MOPEN);
 		p = NULL;
 		break;
 	  case MCLOSE + 1:
@@ -1537,7 +1570,7 @@ regprop(op)
 	  case MCLOSE + 7:
 	  case MCLOSE + 8:
 	  case MCLOSE + 9:
-		sprintf(buf + strlen(buf), "MCLOSE%d", OP(op) - MCLOSE);
+		sprintf(buf + STRLEN(buf), "MCLOSE%d", OP(op) - MCLOSE);
 		p = NULL;
 		break;
 	  case BACKREF + 1:
@@ -1549,7 +1582,7 @@ regprop(op)
 	  case BACKREF + 7:
 	  case BACKREF + 8:
 	  case BACKREF + 9:
-		sprintf(buf + strlen(buf), "BACKREF%d", OP(op) - BACKREF);
+		sprintf(buf + STRLEN(buf), "BACKREF%d", OP(op) - BACKREF);
 		p = NULL;
 		break;
 	  case STAR:
@@ -1559,7 +1592,7 @@ regprop(op)
 		p = "PLUS";
 		break;
 	  default:
-		sprintf(buf + strlen(buf), "corrupt %d", OP(op));
+		sprintf(buf + STRLEN(buf), "corrupt %d", OP(op));
 		p = NULL;
 		break;
 	}
@@ -1583,11 +1616,11 @@ regprop(op)
 
 static int
 strcspn(s1, s2)
-	const char		   *s1;
-	const char		   *s2;
+	const char_u		   *s1;
+	const char_u		   *s2;
 {
-	register char  *scan1;
-	register char  *scan2;
+	register char_u  *scan1;
+	register char_u  *scan2;
 	register int	count;
 
 	count = 0;
@@ -1605,47 +1638,32 @@ strcspn(s1, s2)
  * Compare two strings, ignore case if reg_ic set.
  * Return 0 if strings match, non-zero otherwise.
  */
-	static int
+	int
 cstrncmp(s1, s2, n)
-	char		   *s1, *s2;
+	char_u		   *s1, *s2;
 	int 			n;
 {
 	if (!reg_ic)
-		return strncmp(s1, s2, (size_t)n);
+		return STRNCMP(s1, s2, (size_t)n);
 
-#ifndef UNIX
-	return strnicmp(s1, s2, (size_t)n);
-#else
-# ifdef STRNCASECMP
-	return strncasecmp(s1, s2, (size_t)n);
-# else
-	while (n && *s1 && *s2)
-	{
-		if (mkup(*s1) != mkup(*s2))
-			return 1;
-		s1++;
-		s2++;
-		n--;
-	}
-	if (n)
-		return 1;
-	return 0;
-# endif	/* STRNCASECMP */
-#endif	/* UNIX */
+	return vim_strnicmp(s1, s2, (size_t)n);
 }
 
-	char *
+	char_u *
 cstrchr(s, c)
-	char		   *s;
+	char_u		   *s;
 	int				c;
 {
-	char		   *p;
+	char_u		   *p;
 
-	c = mkup(c);
+	if (!reg_ic)
+		return STRCHR(s, c);
+
+	c = TO_UPPER(c);
 
 	for (p = s; *p; p++)
 	{
-		if (mkup(*p) == c)
+		if (TO_UPPER(*p) == c)
 			return p;
 	}
 	return NULL;
