@@ -1,11 +1,9 @@
 /* vi:ts=4:sw=4
  *
- * term.h -- VIM - Vi IMproved
+ * VIM - Vi IMproved		by Bram Moolenaar
  *
- * Code Contributions By:	Bram Moolenaar			mool@oce.nl
- *							Tim Thompson			twitch!tjt
- *							Tony Andrews			onecom!wldrdg!tony 
- *							G. R. (Fred) Walter		watmath!watcgl!grwalter 
+ * Read the file "credits.txt" for a list of people who contributed.
+ * Read the file "uganda.txt" for copying and usage conditions.
  */
 
 /*
@@ -16,6 +14,13 @@
  * and works around the deficiency, if necessary.
  */
 
+#ifdef SASC
+/*
+ * the SAS C compiler has a bug that makes typedefs being forget sometimes
+ */
+typedef unsigned char char_u;
+#endif
+
 /*
  * the terminal capabilities are stored in this structure
  * keep in sync with array in term.c
@@ -23,57 +28,60 @@
 typedef struct _tcarr
 {
 /* output codes */
-  char *t_name;		/* name of this terminal entry */
-  char *t_el;		/* el       ce	clear to end of line */
-  char *t_il;		/* il1      al	add new blank line */
-  char *t_cil;		/* il       AL	add number of blank lines */
-  char *t_dl;		/* dl1      dl	delete line */
-  char *t_cdl;		/* dl       DL	delete number of lines */
-  char *t_ed;		/* clear    cl	clear screen */
-  char *t_ci;		/* civis    vi	cursur invisible */
-  char *t_cv;		/* cnorm    ve	cursur visible */
-  char *t_tp;		/* sgr0     me	normal mode */
-  char *t_ti;		/* rev      mr	reverse mode */
-  char *t_cm;		/* cup      cm	cursor motion */
-  char *t_sr;		/* ri       sr	scroll reverse (backward) */
-  char *t_cri;		/* cuf      RI	cursor number of chars right */
-  char *t_vb;		/* flash    vb	visual bell */
-  char *t_ks;		/* smkx     ks	put terminal in "keypad transmit" mode */
-  char *t_ke;		/* rmkx     ke	out of "keypad transmit" mode */
-  char *t_ts;		/*          ti	put terminal in termcap mode */
-  char *t_te;		/*          te	out of termcap mode */
+  char_u *t_name;	/* name of this terminal entry */
+  char_u *t_el;		/* el       ce	clear to end of line */
+  char_u *t_il;		/* il1      al	add new blank line */
+  char_u *t_cil;	/* il       AL	add number of blank lines */
+  char_u *t_dl;		/* dl1      dl	delete line */
+  char_u *t_cdl;	/* dl       DL	delete number of lines */
+  char_u *t_ed;		/* clear    cl	clear screen */
+  char_u *t_ci;		/* civis    vi	cursur invisible */
+  char_u *t_cv;		/* cnorm    ve	cursur visible */
+  char_u *t_tp;		/* sgr0     me	normal mode */
+  char_u *t_ti;		/* rev      mr	reverse mode */
+  char_u *t_se;		/* rmso     se	normal mode */
+  char_u *t_so;		/* smso     so	standout mode */
+  char_u *t_ms;		/* msgr     ms	save to move cursor in reverse mode */
+  char_u *t_cm;		/* cup      cm	cursor motion */
+  char_u *t_sr;		/* ri       sr	scroll reverse (backward) */
+  char_u *t_cri;	/* cuf      RI	cursor number of chars right */
+  char_u *t_vb;		/* flash    vb	visual bell */
+  char_u *t_ks;		/* smkx     ks	put terminal in "keypad transmit" mode */
+  char_u *t_ke;		/* rmkx     ke	out of "keypad transmit" mode */
+  char_u *t_ts;		/*          ti	put terminal in termcap mode */
+  char_u *t_te;		/*          te	out of termcap mode */
 
 /* key codes */
-  char *t_ku;		/* kcuu1    ku	arrow up */
-  char *t_kd;		/* kcud1    kd	arrow down */
-  char *t_kl;		/* kcub1    kl	arrow left */
-  char *t_kr;		/* kcuf1    kr	arrow right */
-  char *t_sku;		/* shift arrow up */
-  char *t_skd;		/* shift arrow down */
-  char *t_skl;		/* kLFT     #4	shift arrow left */
-  char *t_skr;		/* kRIT     %	shift arrow right */
-  char *t_f1;		/* kf1      k1	function key 1 */
-  char *t_f2;		/* kf2      k2	function key 2 */
-  char *t_f3;		/* kf3      k3	function key 3 */
-  char *t_f4;		/* kf4      k4	function key 4 */
-  char *t_f5;		/* kf5      k5	function key 5 */
-  char *t_f6;		/* kf6      k6	function key 6 */
-  char *t_f7;		/* kf7      k7	function key 7 */
-  char *t_f8;		/* kf8      k8	function key 8 */
-  char *t_f9;		/* kf9      k9	function key 9 */
-  char *t_f10;		/* kf10     k;	function key 10 */
-  char *t_sf1;		/* kf11     F1	shifted function key 1 */
-  char *t_sf2;		/* kf12     F2	shifted function key 2 */
-  char *t_sf3;		/* kf13     F3	shifted function key 3 */
-  char *t_sf4;		/* kf14     F4	shifted function key 4 */
-  char *t_sf5;		/* kf15     F5	shifted function key 5 */
-  char *t_sf6;		/* kf16     F6	shifted function key 6 */
-  char *t_sf7;		/* kf17     F7	shifted function key 7 */
-  char *t_sf8;		/* kf18     F8	shifted function key 8 */
-  char *t_sf9;		/* kf19     F9	shifted function key 9 */
-  char *t_sf10;		/* kf20     FA	shifted function key 10 */
-  char *t_help;		/* khlp     %1	help key */
-  char *t_undo;		/* kund     &8	undo key */
+  char_u *t_ku;		/* kcuu1    ku	arrow up */
+  char_u *t_kd;		/* kcud1    kd	arrow down */
+  char_u *t_kl;		/* kcub1    kl	arrow left */
+  char_u *t_kr;		/* kcuf1    kr	arrow right */
+  char_u *t_sku;	/* shift arrow up */
+  char_u *t_skd;	/* shift arrow down */
+  char_u *t_skl;	/* kLFT     #4	shift arrow left */
+  char_u *t_skr;	/* kRIT     %	shift arrow right */
+  char_u *t_f1;		/* kf1      k1	function key 1 */
+  char_u *t_f2;		/* kf2      k2	function key 2 */
+  char_u *t_f3;		/* kf3      k3	function key 3 */
+  char_u *t_f4;		/* kf4      k4	function key 4 */
+  char_u *t_f5;		/* kf5      k5	function key 5 */
+  char_u *t_f6;		/* kf6      k6	function key 6 */
+  char_u *t_f7;		/* kf7      k7	function key 7 */
+  char_u *t_f8;		/* kf8      k8	function key 8 */
+  char_u *t_f9;		/* kf9      k9	function key 9 */
+  char_u *t_f10;	/* kf10     k;	function key 10 */
+  char_u *t_sf1;	/* kf11     F1	shifted function key 1 */
+  char_u *t_sf2;	/* kf12     F2	shifted function key 2 */
+  char_u *t_sf3;	/* kf13     F3	shifted function key 3 */
+  char_u *t_sf4;	/* kf14     F4	shifted function key 4 */
+  char_u *t_sf5;	/* kf15     F5	shifted function key 5 */
+  char_u *t_sf6;	/* kf16     F6	shifted function key 6 */
+  char_u *t_sf7;	/* kf17     F7	shifted function key 7 */
+  char_u *t_sf8;	/* kf18     F8	shifted function key 8 */
+  char_u *t_sf9;	/* kf19     F9	shifted function key 9 */
+  char_u *t_sf10;	/* kf20     FA	shifted function key 10 */
+  char_u *t_help;	/* khlp     %1	help key */
+  char_u *t_undo;	/* kund     &8	undo key */
   /* adjust inchar() for last entry! */
 } Tcarr;
 
@@ -92,6 +100,9 @@ extern Tcarr term_strings;	/* currently used terminal strings */
 #define T_CV	(term_strings.t_cv)
 #define T_TP	(term_strings.t_tp)
 #define T_TI	(term_strings.t_ti)
+#define T_SE	(term_strings.t_se)
+#define T_SO	(term_strings.t_so)
+#define T_MS	(term_strings.t_ms)
 #define T_CM	(term_strings.t_cm)
 #define T_SR	(term_strings.t_sr)
 #define T_CRI	(term_strings.t_cri)
@@ -124,6 +135,9 @@ extern Tcarr term_strings;	/* currently used terminal strings */
 \0\033[1 p\0\
 \0\033[0m\0\
 \0\033[7m\0\
+\0\033[0m\0\
+\0\033[33m\0\
+\0\001\0\
 \0\033[%i%d;%dH\0\
 \1\033[%dC\0\
 \5\233A\0\
@@ -175,6 +189,7 @@ extern Tcarr term_strings;	/* currently used terminal strings */
 \0\033[2J\0\
 \2\033[0m\0\
 \0\033[7m\0\
+\2\001\0\
 \0\033[%i%d;%dH\0\
 \1\033[%dC\0\
 \0\0"
@@ -192,6 +207,7 @@ extern Tcarr term_strings;	/* currently used terminal strings */
 \1\033[2J\0\
 \2\033[0m\0\
 \0\033[7m\0\
+\2\001\0\
 \0\033[%i%d;%dH\0\
 \1\033[%dC\0\
 \5\316H\0\
@@ -233,6 +249,9 @@ extern Tcarr term_strings;	/* currently used terminal strings */
 \1\033|J\0\
 \2\033|0m\0\
 \0\033|79m\0\
+\0\033|0m\0\
+\0\033|20m\0\
+\0\001\0\
 \0\033|%i%d;%dH\0\
 \7\316H\0\
 \0\316P\0\
@@ -262,6 +281,55 @@ extern Tcarr term_strings;	/* currently used terminal strings */
 \0\316]\0\
 \0\0"
 
+/*
+ * These codes are valid for the NT Console 
+ * The entries that start with ESC | are translated into console calls 
+ * in winnt.c.
+ */
+#  define NTCONSOLE_TCAP "ntconsole\0\
+\0\033|K\0\
+\0\033|L\0\
+\0\033|%dL\0\
+\0\033|M\0\
+\0\033|%dM\0\
+\0\033|J\0\
+\0\033|v\0\
+\0\033|V\0\
+\0\033|0m\0\
+\0\033|79m\0\
+\0\033|0m\0\
+\0\033|20m\0\
+\0\001\0\
+\0\033|%i%d;%dH\0\
+\7\316H\0\
+\0\316P\0\
+\0\316K\0\
+\0\316M\0\
+\2\316s\0\
+\0\316t\0\
+\0\316;\0\
+\0\316<\0\
+\0\316=\0\
+\0\316>\0\
+\0\316?\0\
+\0\316@\0\
+\0\316A\0\
+\0\316B\0\
+\0\316C\0\
+\0\316D\0\
+\0\316T\0\
+\0\316U\0\
+\0\316V\0\
+\0\316W\0\
+\0\316X\0\
+\0\316Y\0\
+\0\316Z\0\
+\0\316[\0\
+\0\316\\\0\
+\0\316]\0\
+\0\0"
+
+
 #  define VT52_TCAP "vt52\0\
 \0\033K\0\
 \0\033T\0\
@@ -269,6 +337,7 @@ extern Tcarr term_strings;	/* currently used terminal strings */
 \1\014\0\
 \2\033SO\0\
 \0\033S2\0\
+\2\001\0\
 \0\033Y%+ %+ \0\
 \0\0"
 
@@ -285,6 +354,7 @@ extern Tcarr term_strings;	/* currently used terminal strings */
 \0\033[H\033[2J\0\
 \2\033[m\0\
 \0\033[7m\0\
+\2\001\0\
 \0\033[%i%d;%dH\0\
 \0\033M\0\
 \0\033[%dC\0\
@@ -333,6 +403,9 @@ extern Tcarr term_strings;	/* currently used terminal strings */
 \0[CV]\0\
 \0[TP]\0\
 \0[TI]\0\
+\0[SE]\0\
+\0[SO]\0\
+\0[MS]\0\
 \0[%dCM%d]\0\
 \0[SR]\0\
 \0[CRI%d]\0\
@@ -381,9 +454,13 @@ extern Tcarr term_strings;	/* currently used terminal strings */
 #   define DFLT_TCAP AMIGA_TCAP
 #  endif /* AMIGA */
 
-#  ifdef MSDOS
-#   define DFLT_TCAP PCTERM_TCAP
-#  endif /* MSDOS */
+#  ifdef NT
+#   define DFLT_TCAP NTCONSOLE_TCAP
+#  else
+#   ifdef MSDOS
+#    define DFLT_TCAP PCTERM_TCAP                       
+#   endif /* MSDOS */                                   
+#  endif /* NT */ 
 
 #  ifdef UNIX
 #   define DFLT_TCAP ANSI_TCAP
@@ -392,7 +469,7 @@ extern Tcarr term_strings;	/* currently used terminal strings */
 # else /* NO_BUILTIN_TCAPS */
 #  define DUMB_TCAP "dumb\0\
 \5\014\0\
-\4\033[%i%d;%dH\0\
+\7\033[%i%d;%dH\0\
 \0\0"
 # endif /* NO_BUILTIN_TCAPS */
 
@@ -418,6 +495,9 @@ extern Tcarr term_strings;	/* currently used terminal strings */
 \0\033[1 p\0\
 \0\033[0m\0\
 \0\033[7m\0\
+\0\033[0m\0\
+\0\033[33m\0\
+\0\001\0\
 \0\033[%i%p1%d;%p2%dH\0\
 \1\033[%p1%dC\0\
 \5\233A\0\
@@ -469,6 +549,7 @@ extern Tcarr term_strings;	/* currently used terminal strings */
 \0\033[2J\0\
 \2\033[0m\0\
 \0\033[7m\0\
+\2\001\0\
 \0\033[%i%p1%d;%p2%dH\0\
 \1\033[%p1%dC\0\
 \0\0"
@@ -486,6 +567,7 @@ extern Tcarr term_strings;	/* currently used terminal strings */
 \1\033[2J\0\
 \2\033[0m\0\
 \0\033[7m\0\
+\2\001\0\
 \0\033[%i%p1%d;%p2%dH\0\
 \1\033[%p1%dC\0\
 \5\316H\0\
@@ -527,6 +609,9 @@ extern Tcarr term_strings;	/* currently used terminal strings */
 \1\033|J\0\
 \2\033|0m\0\
 \0\033|79m\0\
+\0\033|0m\0\
+\0\033|20m\0\
+\0\001\0\
 \0\033|%i%p1%d;%p2%dH\0\
 \7\316H\0\
 \0\316P\0\
@@ -556,6 +641,55 @@ extern Tcarr term_strings;	/* currently used terminal strings */
 \0\316]\0\
 \0\0"
 
+/*
+ * These codes are valid for the NT Console 
+ * The entries that start with ESC | are translated into console calls 
+ * in winnt.c.
+ */
+#  define NTCONSOLE_TCAP "ntconsole\0\
+\0\033|K\0\
+\0\033|L\0\
+\0\033|%dL\0\
+\0\033|M\0\
+\0\033|%dM\0\
+\0\033|J\0\
+\0\033|v\0\
+\0\033|V\0\
+\0\033|0m\0\
+\0\033|79m\0\
+\0\033|0m\0\
+\0\033|20m\0\
+\0\001\0\
+\0\033|%i%p1%d;%p2%dH\0\
+\7\316H\0\
+\0\316P\0\
+\0\316K\0\
+\0\316M\0\
+\2\316s\0\
+\0\316t\0\
+\0\316;\0\
+\0\316<\0\
+\0\316=\0\
+\0\316>\0\
+\0\316?\0\
+\0\316@\0\
+\0\316A\0\
+\0\316B\0\
+\0\316C\0\
+\0\316D\0\
+\0\316T\0\
+\0\316U\0\
+\0\316V\0\
+\0\316W\0\
+\0\316X\0\
+\0\316Y\0\
+\0\316Z\0\
+\0\316[\0\
+\0\316\\\0\
+\0\316]\0\
+\0\0"
+
+
 #  define VT52_TCAP "vt52\0\
 \0\033K\0\
 \0\033T\0\
@@ -563,6 +697,7 @@ extern Tcarr term_strings;	/* currently used terminal strings */
 \1\014\0\
 \2\033SO\0\
 \0\033S2\0\
+\2\001\0\
 \0\033Y%+ %+ \0\
 \0\0"
 
@@ -579,6 +714,7 @@ extern Tcarr term_strings;	/* currently used terminal strings */
 \0\033[H\033[2J\0\
 \2\033[m\0\
 \0\033[7m\0\
+\2\001\0\
 \0\033[%i%p1%d;%p2%dH\0\
 \0\033M\0\
 \0\033[%p1%dC\0\
@@ -627,6 +763,9 @@ extern Tcarr term_strings;	/* currently used terminal strings */
 \0[CV]\0\
 \0[TP]\0\
 \0[TI]\0\
+\0[SE]\0\
+\0[SO]\0\
+\0[MS]\0\
 \0[%p1%dCM%p2%d]\0\
 \0[SR]\0\
 \0[CRI%p1%d]\0\
@@ -675,9 +814,13 @@ extern Tcarr term_strings;	/* currently used terminal strings */
 #   define DFLT_TCAP AMIGA_TCAP
 #  endif /* AMIGA */
 
-#  ifdef MSDOS
-#   define DFLT_TCAP PCTERM_TCAP
-#  endif /* MSDOS */
+#  ifdef NT
+#   define DFLT_TCAP NTCONSOLE_TCAP
+#  else
+#   ifdef MSDOS
+#    define DFLT_TCAP PCTERM_TCAP                       
+#   endif /* MSDOS */                                   
+#  endif /* NT */ 
 
 #  ifdef UNIX
 #   define DFLT_TCAP ANSI_TCAP
@@ -689,7 +832,7 @@ extern Tcarr term_strings;	/* currently used terminal strings */
  */
 #  define DUMB_TCAP "dumb\0\
 \5\014\0\
-\4\033[%i%p1%d;%p2%dH\0\
+\7\033[%i%p1%d;%p2%dH\0\
 \0\0"
 # endif /* NO_BUILTIN_TCAPS */
 
